package zero1hd.rhythmbullet.audio.visualizer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Pixmap.Format;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.utils.Array;

import zero1hd.rhythmbullet.audio.MusicManager;

public class BasicVisualizer extends VisualizerCore {
	private Pixmap pixmap;
	private Texture bar;
	private int barWidth;
	private int binsPerBar;
	private int spaceBetweenBars;
	private Texture[] textures;
	private Sprite[] bars;
	private int smoothRange;
	private float barHeightMultiplier;
	private float rotation;
	private Vector2 angleRot;
	private boolean flip;
	private Array<MirrorVisualizer> mirrors;
	private boolean reverse;
	
	public BasicVisualizer() {
		super(Gdx.graphics.getWidth(), Gdx.graphics.getHeight()/2, 0, 0);
		mirrors = new Array<>();
		barHeightMultiplier = Gdx.graphics.getHeight()*0.01f;
		pixmap = new Pixmap(2, 2, Format.RGBA8888);
		pixmap.setColor(Color.WHITE);
		pixmap.fill();
		barCount = 70;
		smoothRange = 4;
		angleRot = new Vector2(MathUtils.cosDeg(rotation), MathUtils.sinDeg(rotation));
		textures = new Texture[barCount];
		bars = new Sprite[barCount];
		spaceBetweenBars = 4;

		for (int i = 0; i < textures.length; i++) {
			textures[i] = new Texture(pixmap);
			bars[i] = new Sprite(textures[i]);
		}
		
		updatePositionInfo();
		pixmap.dispose();
	}
	
	@Override
	public void render(Batch batch, float parentAlpha) {
		if (mm != null) {
			//Averaging bins together
			for (int i = 0; i < barCount; i++) {
				bars[i].setSize(barWidth, 0);
				float barHeight = 0;
				for (int j = 0; j < binsPerBar; j++) {
					barHeight += Math.abs(audioPCM[j+i*binsPerBar]);
				}
				barHeight /= binsPerBar;
				barHeight *= barHeightMultiplier;
				bars[i].setSize(barWidth, barHeight);
			}
			
			//Drawing...
			for (int i = 0; i < barCount; i++) {
				int avg = 0;
				//Averaging the bars
				for (int range = 0; range < smoothRange; range++) {
					if (i+range < barCount) {
						avg += bars[i+range].getHeight();
					}
					if (i-range >= 0) {
						avg += bars[i-range].getHeight();
					}
				}
				bars[i].setSize(barWidth, avg/(smoothRange*2));
				bars[i].draw(batch);
			}
			
			for (int i = 0; i < mirrors.size; i++) {
				mirrors.get(i).render(batch, parentAlpha, bars);
			}
		}
		super.render(batch, parentAlpha);
	}
	
	@Override
	public void setMM(MusicManager mdp) {
		super.setMM(mdp);
		float validBins = (5000/((mdp.getSampleRate()/2)/((audioPCM.length/2)+1)));
		Gdx.app.debug("Visualizer", "valid frequency bins " + validBins);
		binsPerBar = MathUtils.round((validBins/barCount));
	}
	
	@Override
	public void dispose() {
		bar.dispose();
		super.dispose();
	}
	
	public void updatePositionInfo() {
		int barSpace = 0;
		angleRot.set(MathUtils.cosDeg(rotation), MathUtils.sinDeg(rotation));
		
		barWidth = MathUtils.ceil((float) width/(float) barCount);
		barWidth -= spaceBetweenBars;
		
		for (int i = 0; i < bars.length; i++) {
			barSpace = i*(barWidth+spaceBetweenBars);
			if (flip) {
				bars[i].setRotation(rotation+180);
			} else {
				bars[i].setRotation(rotation);
			}
			if (reverse) {
				bars[bars.length-i-1].setPosition(xPos + barSpace*angleRot.x, yPos + barSpace*angleRot.y);
			} else {
				bars[i].setPosition(xPos + barSpace*angleRot.x, yPos + barSpace*angleRot.y);
			}
		}
		for (int i = 0; i < mirrors.size; i++) {
			mirrors.get(i).position(barWidth, spaceBetweenBars);
		}
	}
	
	public void setColor(Color color) {
		for (int i = 0; i < bars.length; i++) {
			bars[i].setColor(color);
		}
	}
	
	public void setColor(float r, float g, float b, float a) {
		for (int i = 0; i < bars.length; i++) {
			bars[i].setColor(r, g, b, a);
		}
	}
	
	public void setRotation(float rotation) {
		this.rotation = rotation;
	}
	
	public void flip() {
		flip = flip ? false : true;
	}
	
	public boolean isFlipped() {
		return flip;
	}
	
	public void addMirrorVisualizer(MirrorVisualizer mirror) {
		mirror.setup(bars, xPos, yPos, rotation);
		mirrors.add(mirror);
	}
	
	public void removeMirrorVisualizer(MirrorVisualizer mirror) {
		mirrors.removeValue(mirror, true);
	}
	
	public void setSpaceBetweenBars(int spaceBetweenBars) {
		this.spaceBetweenBars = spaceBetweenBars;
	}
	
	public int getBarWidth() {
		return barWidth;
	}
	
	public void reverse() {
		reverse = reverse ? false : true;
	}
	
	public boolean isReversed() {
		return reverse;
	}
}
